use crate::instruction::{PRegNum, UnitKind, UnitNum};
use fayalite::prelude::*;

#[derive(Clone, Eq, PartialEq, Hash, Debug)]
#[non_exhaustive]
pub struct CpuConfig {
    pub units: Vec<UnitKind>,
    pub out_reg_num_width: usize,
}

impl CpuConfig {
    pub fn unit_num_width(&self) -> usize {
        UInt::range(0..self.units.len()).width()
    }
    pub fn unit_num(&self) -> UnitNum<DynSize> {
        UnitNum[self.unit_num_width()]
    }
    pub fn p_reg_num(&self) -> PRegNum<DynSize, DynSize> {
        PRegNum[self.unit_num_width()][self.out_reg_num_width]
    }
}
